/*
 * Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms
 * and conditions in the IBM International Program License Agreement.
 *
 *  Copyright IBM Corporation 2003, 2004. All Rights Reserved. 
 */
package com.ibm.xtools.modeler.ui.pde.examples.counter.actions;

import java.lang.reflect.InvocationTargetException;
import java.text.MessageFormat;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import com.ibm.xtools.emf.msl.ResourceSetReadOperation;
import com.ibm.xtools.modeler.UMLModeler;
import com.ibm.xtools.modeler.ui.pde.examples.counter.CounterPlugin;

/**
 * This action delegate counts the number of modeling elements owned by the
 * selected element.
 * 
 * @see IWorkbenchWindowActionDelegate
 */
public class ElementCount
	implements IWorkbenchWindowActionDelegate {

	/**
	 * The shell this action delegate was initialized for
	 */
	private Shell shell;

	/**
	 * Key for title of message dialog
	 */
	private static final String KEY_TITLE = "title"; //$NON-NLS-1$

	/**
	 * Key for message to output
	 */
	private static final String KEY_MESSAGE = "message"; //$NON-NLS-1$

	/**
	 * Key for message to output when an exception occurred
	 */
	private static final String KEY_EXCEPTION = "message.exception"; //$NON-NLS-1$

	/**
	 * Key for message to output when the operation was cancelled
	 */
	private static final String KEY_INTERRUPTED = "message.interrupted"; //$NON-NLS-1$

	/**
	 * Constructor
	 */
	public ElementCount() {
		//No-op
	}

	/**
	 * Counts number of modeling elements in selection
	 * 
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {

		String strTitle = CounterPlugin.getResourceString(KEY_TITLE);

		try {
			final String strMessage[] = {null};

			UMLModeler.getEditingDomain().run(new ResourceSetReadOperation() {

				protected void execute(IProgressMonitor monitor) {

					int count = 0;

					// Get selection from the window
					List elements = UMLModeler.getUMLUIHelper()
						.getSelectedElements();

					for (Iterator i = elements.iterator(); i.hasNext();) {
						EObject eObject = (EObject) i.next();

						// Accumulate number of owned element
						count += countOwnedContents(eObject);
					}

					// Output results
					strMessage[0] = MessageFormat.format(
					CounterPlugin.getResourceString(KEY_MESSAGE),
						new Object[] {String.valueOf(count)});

				}
			}, new NullProgressMonitor());
			
			MessageDialog.openInformation(shell, strTitle,
				strMessage[0]);

		} catch (InvocationTargetException e) {
			
			String strMessage = CounterPlugin.getResourceString(KEY_EXCEPTION);
			MessageDialog.openInformation(shell, strTitle,
				strMessage);
			throw new RuntimeException(e);
			
		} catch (InterruptedException e) {
			
			String strMessage  = CounterPlugin.getResourceString(KEY_INTERRUPTED);
			MessageDialog.openInformation(shell, strTitle,
				strMessage);
		}

	}

	/**
	 * Calculates the number of modeling elements owned by the specified object
	 * 
	 * @param eObject
	 *            The modeling element to calculate the number of owned elements
	 *            from
	 * @return The number of owned modeling element
	 */
	private int countOwnedContents(final EObject eObject) {

		int count = 0;

		TreeIterator contents = eObject.eAllContents();

		while (contents.hasNext()) {
			contents.next();
			count++;
		}

		return count;
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		//No-op
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
		//No-op
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow window) {
		shell = window.getShell();
	}
}