/*
 * Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms
 * and conditions in the IBM International Program License Agreement.
 *
 *  Copyright IBM Corporation 2003, 2004. All Rights Reserved. 
 */
package com.ibm.xtools.modeler.ui.pde.examples.events;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.console.MessageConsoleStream;

import com.ibm.xtools.emf.msl.IOperationEvent;
import com.ibm.xtools.emf.msl.OperationListener;
import com.ibm.xtools.modeler.UMLModeler;

/**
 * An Operation Event Listener that logs events to the console
 */
public class OperationEventLogger {

	/**
	 * Key for name of console
	 */
	private static final String KEY_CONSOLE_NAME = "consoleName"; //$NON-NLS-1$

	/**
	 * Path for console icon
	 */
	private static final String CONSOLE_ICON_PATH = "icons/events.gif"; //$NON-NLS-1$

	/**
	 * Key for Done
	 */
	private static final String KEY_DONE = "message.done"; //$NON-NLS-1$

	/**
	 * Key for Undone
	 */
	private static final String KEY_UNDONE = "message.undone"; //$NON-NLS-1$

	/**
	 * Key for Redone
	 */
	private static final String KEY_REDONE = "message.redone"; //$NON-NLS-1$

	/**
	 * Key for Start Listening
	 */
	private static final String KEY_START_LISTENING = "message.startListening"; //$NON-NLS-1$

	/**
	 * Key for Stop Listening
	 */
	private static final String KEY_STOP_LISTENING = "message.stopListening"; //$NON-NLS-1$

	/**
	 * Console
	 */
	MessageConsole console = null;

	/**
	 * Console stream
	 */
	private MessageConsoleStream consoleStream;

	/**
	 * Log flag
	 */
	private boolean listen = false;

	/**
	 * Listener
	 */
	private OperationListener operationListener = null;

	public OperationEventLogger() {
		// No-op
	}

	public boolean isListening() {
		return listen;
	}

	public void startListening() {
		// Show console
		IConsoleManager conMgr = ConsolePlugin.getDefault().getConsoleManager();
		conMgr.showConsoleView(getConsole());

		// Register listener
		UMLModeler.getEditingDomain().addOperationListener(
			getOperationListener());

		consoleStream.println(EventsPlugin
			.getResourceString(KEY_START_LISTENING));

		listen = true;
	}

	public void stopListening() {
		listen = false;

		consoleStream.println(EventsPlugin
			.getResourceString(KEY_STOP_LISTENING));

		// Register listener
		UMLModeler.getEditingDomain().removeOperationListener(
			getOperationListener());
	}

	/**
	 * Lazy console creation
	 */
	private IConsole getConsole() {

		if (null == console) {

			ImageDescriptor imageDesc = EventsPlugin
				.imageDescriptorFromPlugin(EventsPlugin.getDefault()
					.getBundle().getSymbolicName(), CONSOLE_ICON_PATH);

			console = new MessageConsole(EventsPlugin
				.getResourceString(KEY_CONSOLE_NAME), imageDesc);

			IConsoleManager conMgr = ConsolePlugin.getDefault()
				.getConsoleManager();

			conMgr.addConsoles(new IConsole[] {console});

			consoleStream = console.newMessageStream();
		}

		return console;
	}

	/**
	 * Lazy OperationListener creation
	 */
	private OperationListener getOperationListener() {
		if (null == operationListener) {
			operationListener = new OperationListener() {

				public void done(IOperationEvent event) {
					logNotifications(EventsPlugin.getResourceString(KEY_DONE),
						event.getNotifications());
				}

				public void undone(IOperationEvent event) {
					logNotifications(EventsPlugin.getResourceString(KEY_UNDONE),
						event.getNotifications());
				}

				public void redone(IOperationEvent event) {
					logNotifications(EventsPlugin.getResourceString(KEY_REDONE),
						event.getNotifications());
				}
			};
		}

		return operationListener;
	}

	/**
	 * Logs the specified notifications
	 * 
	 * @param heading
	 *            The heading to output before events
	 * @param notifications
	 *            The notifications to log
	 */
	private void logNotifications(String heading, List notifications) {

		consoleStream.println(heading);

		for (Iterator iter = notifications.iterator(); iter.hasNext();) {
			Notification notification = (Notification) iter.next();
			log(notification);
		}
	}

	/**
	 * Logs the specified notification
	 * 
	 * @param notificatoin
	 *            The notification to log
	 */
	private void log(Notification notification) {

		if (null != notification) {
			consoleStream.println("        " + notification.toString());  //$NON-NLS-1$
		}
	}

}