/*
 * Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms
 * and conditions in the IBM International Program License Agreement.
 *
 *  Copyright IBM Corporation 2003, 2004. All Rights Reserved. 
 */
package com.ibm.xtools.modeler.ui.pde.examples.profile.actions;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.emf.common.util.URI;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.uml2.LiteralBoolean;
import org.eclipse.uml2.Model;
import org.eclipse.uml2.Profile;
import org.eclipse.uml2.Property;
import org.eclipse.uml2.Stereotype;
import org.eclipse.uml2.UML2Package;
import org.eclipse.uml2.VisibilityKind;

import com.ibm.xtools.emf.msl.ResourceSetModifyOperation;
import com.ibm.xtools.modeler.UMLModeler;
import com.ibm.xtools.modeler.ui.pde.examples.profile.ProfilePlugin;

/**
 * Generates a profile and a model, apply the profile to the model, and
 * associate stereotypes with some of the model's elements
 * 
 * @see IWorkbenchWindowActionDelegate
 */
public class CreateAndApplyProfile
	implements IWorkbenchWindowActionDelegate {

	private static final String KEY_GENERATED_CLASS_NAME = "generated.class.name"; //$NON-NLS-1$

	private static final String KEY_GENERATED_MODEL_NAME = "generated.model.name"; //$NON-NLS-1$

	private static final String KEY_GENERATED_MODEL_FILENAME = "generated.model.filename"; //$NON-NLS-1$

	private static final String KEY_GENERATED_STEREOTYPE_ATTRIBUTE_NAME = "generated.stereotype.attribute.name"; //$NON-NLS-1$

	private static final String KEY_GENERATED_STEREOTYPE_NAME = "generated.stereotype.name"; //$NON-NLS-1$

	private static final String KEY_GENERATED_PROFILE_NAME = "generated.profile.name"; //$NON-NLS-1$

	private static final String KEY_GENERATED_PROFILE_FILENAME = "generated.profile.filename"; //$NON-NLS-1$

	private static final String KEY_GENERATED_PROJECT_NAME = "generated.project.name"; //$NON-NLS-1$

	private static final String STEREOTYPE_ATTRIBUTE_TYPE = "Boolean"; //$NON-NLS-1$

	private static final String EXTENSION_METACLASS = "Class"; //$NON-NLS-1$

	private static final String URI_UML2_PRIMITIVE_TYPE_LIBRARY = "pathmap://UML2_LIBRARIES/UML2PrimitiveTypes.library.uml2"; //$NON-NLS-1$

	private static final String URI_UML2_METAMODEL = "pathmap://UML2_METAMODELS/UML2.metamodel.uml2"; //$NON-NLS-1$

	/**
	 * Title of profile creation operation
	 */
	private static final String KEY_PROFILE_CREATION_OPERATION_TITLE = "profileCreationOperationTitle"; //$NON-NLS-1$

	/**
	 * Constructor
	 */
	public CreateAndApplyProfile() {
		//No-op
	}

	/**
	 * Does the work
	 * 
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	public void run(IAction action) {

		final IProject project[] = {null};
		final Profile profile[] = {null};
		final Model model[] = {null};

		try {
			/* Create project */
			ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {

				public void run(IProgressMonitor monitor)
					throws CoreException {
					project[0] = ResourcesPlugin.getWorkspace().getRoot()
						.getProject(
							ProfilePlugin
								.getResourceString(KEY_GENERATED_PROJECT_NAME));

					if (false == project[0].exists()) {
						project[0].create(monitor);
					}

					project[0].open(monitor);
				}
			}, new NullProgressMonitor());

			/* Perform remaining work within a ResourceSetWriteOperation */
			UMLModeler.getEditingDomain().run(
				new ResourceSetModifyOperation(ProfilePlugin
					.getResourceString(KEY_PROFILE_CREATION_OPERATION_TITLE)) {

					protected void execute(IProgressMonitor monitor)
						throws InvocationTargetException, InterruptedException {

						try {
							/*
							 * Generate profile, generate model, apply one to
							 * the other
							 */
							profile[0] = generateProfile(project[0], monitor);
							model[0] = generateModel(project[0], monitor);
							applyProfileToModel(model[0], profile[0], monitor);

							/* Save */
							UMLModeler.saveProfile(profile[0]);
							UMLModeler.saveModel(model[0]);

						} catch (Exception e) {
							throw new InvocationTargetException(e);
						}
					}
				}, new NullProgressMonitor());

		} catch (Exception e) {
			/* Close profile and model in case of an exception */
			if (null != profile[0]) {
				UMLModeler.closeProfile(profile[0]);
			}

			if (null != model[0]) {
				UMLModeler.closeModel(model[0]);
			}

			throw new RuntimeException(e.getCause());
		}
	}

	/**
	 * Generates a profile containing a single stereotype with a single
	 * attribute
	 * 
	 * @param project
	 *            The project to create the profile into
	 * @param monitor
	 *            A monitor to capture progress
	 * @return The generated profile
	 * @throws IOException
	 *             Could not generate the profile
	 * @throws CoreException
	 *             Could not generate the profile
	 */
	private Profile generateProfile(IProject project, IProgressMonitor monitor)
		throws IOException, CoreException {

		/* Create profile resource */
		IFile file = project.getFile(ProfilePlugin
			.getResourceString(KEY_GENERATED_PROFILE_FILENAME));
		if (file.exists()) {
			file.delete(true, monitor);
		}
		Profile profile = UMLModeler.createProfile(file.getLocation()
			.toOSString());
		profile.setName(ProfilePlugin
			.getResourceString(KEY_GENERATED_PROFILE_NAME));

		/* Create a stereotype that extends the UML Class metaclass */

		/* Add reference from the profile to the UML Class metaclass */
		Model umlMetamodel = UMLModeler.openModel(URI
			.createURI(URI_UML2_METAMODEL));
		org.eclipse.uml2.Class classMetaClass = (org.eclipse.uml2.Class) umlMetamodel
			.getOwnedType(EXTENSION_METACLASS);
		profile.referenceMetaclass(classMetaClass);

		/* Add reference from the profile to the UML2 Primitive Types library */
		Model umlPrimitiveTypesModelLibrary = UMLModeler.openModel(URI
			.createURI(URI_UML2_PRIMITIVE_TYPE_LIBRARY));
		profile.importPackage(VisibilityKind.PUBLIC_LITERAL,
			umlPrimitiveTypesModelLibrary);

		/* Create the Stereotype and add extension relation to the UML2 Class */
		final Stereotype sampleStereotype = (Stereotype) profile
			.createOwnedMember(UML2Package.eINSTANCE.getStereotype());
		sampleStereotype.setName(ProfilePlugin
			.getResourceString(KEY_GENERATED_STEREOTYPE_NAME));

		Property property = sampleStereotype
			.createOwnedAttribute(UML2Package.eINSTANCE.getProperty());
		property.setName(ProfilePlugin
			.getResourceString(KEY_GENERATED_STEREOTYPE_ATTRIBUTE_NAME));
		property.setType(umlPrimitiveTypesModelLibrary
			.getOwnedType(STEREOTYPE_ATTRIBUTE_TYPE));

		LiteralBoolean defaultValue = (LiteralBoolean) property
			.createDefaultValue(UML2Package.eINSTANCE.getLiteralBoolean());
		defaultValue.setValue(false);

		sampleStereotype.createExtension(UML2Package.eINSTANCE.getClass_(),
			false);

		/* Finalize generation of the profile */
		profile.define();

		return profile;
	}

	/**
	 * Generates a model containing a single class
	 * 
	 * @param project
	 *            The project to create the model into
	 * @param monitor
	 *            A monitor to capture progress
	 * @return The generated model
	 * @throws IOException
	 *             Could not generate the model
	 * @throws CoreException
	 *             Could not generate the model
	 */
	private Model generateModel(IProject project, IProgressMonitor monitor)
		throws IOException, CoreException {

		/* Create profile resource */
		IFile file = project.getFile(ProfilePlugin
			.getResourceString(KEY_GENERATED_MODEL_FILENAME));
		if (file.exists()) {
			file.delete(true, monitor);
		}

		Model model = UMLModeler.createModel(file.getLocation().toOSString());
		model
			.setName(ProfilePlugin.getResourceString(KEY_GENERATED_MODEL_NAME));

		/* Create a class */
		org.eclipse.uml2.Class theClass = (org.eclipse.uml2.Class) model
			.createOwnedMember(UML2Package.eINSTANCE.getClass_());
		theClass.setName(ProfilePlugin
			.getResourceString(KEY_GENERATED_CLASS_NAME));

		return model;
	}

	/**
	 * Applies the specified profile to the specified model and applies a
	 * stereotype
	 * 
	 * @param model
	 *            The model to apply the profile to
	 * @param profile
	 *            The profile to apply to the model
	 * @param monitor
	 *            A monitor to capture progress
	 */
	private void applyProfileToModel(Model model, Profile profile,
			IProgressMonitor monitor) {

		/* Apply profile */
		model.apply(profile);

		/* Apply stereotype to a class */
		org.eclipse.uml2.Class sampleClass = (org.eclipse.uml2.Class) model
			.getOwnedMember(ProfilePlugin
				.getResourceString(KEY_GENERATED_CLASS_NAME));

		Stereotype sampleStereotype = profile.getOwnedStereotype(ProfilePlugin
			.getResourceString(KEY_GENERATED_STEREOTYPE_NAME));

		sampleClass.apply(sampleStereotype);

		sampleClass.setValue(sampleStereotype, ProfilePlugin
			.getResourceString(KEY_GENERATED_STEREOTYPE_ATTRIBUTE_NAME),
			Boolean.TRUE);
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#selectionChanged
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		//No-op
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	public void dispose() {
		//No-op
	}

	/**
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	public void init(IWorkbenchWindow windowIn) {
		//No-op
	}
}