/*
 * Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms
 * and conditions in the IBM International Program License Agreement.
 *
 *  Copyright IBM Corporation 2003, 2004. All Rights Reserved. 
 */
package com.ibm.xtools.modeler.ui.pde.examples.validation.constraints;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.Classifier;

import com.ibm.xtools.emf.validation.AbstractModelConstraint;
import com.ibm.xtools.emf.validation.IValidationContext;

/**
 * Example constraint implemented in Java.
 */
public class JavaExample extends AbstractModelConstraint {
	
	/**
	 * Initializes me.
	 */
	public JavaExample() {
		super();
	}

	/**
	 * This constraint checks for generalization cycles on a classifier.
	 * When a cycle is found, the constraint reports the members of the cycle
	 * (excluding the validation target) in
	 * <ul>
	 * <li>the error message.</li>
	 * <li>the results collection of the context.</li>
	 * </ul>
	 */
	public IStatus validate(IValidationContext ctx) {
		final EObject target = ctx.getTarget();

		// extracted to identify the participants in the
		//     generalization cycle
		List cycle = new java.util.ArrayList();

		boolean hasCycle = hasCycle((Classifier)target, cycle);

		if (hasCycle) {
			// the participants in this cycle have been validated, so make sure
			//    that this validation operation does not check them
			//    redundantly
			ctx.skipCurrentConstraintForAll(cycle);

			// the result locus should highlight the members of the cycle in
			//    the model explorer
			ctx.addResults(cycle);

			// including the target in the error message would be redundant,
			//    as it  appears separately in the message
			cycle.remove(target);
		}

		return hasCycle
			? ctx.createFailureStatus(new Object[] {
				target,  // the system puts the element name in the error message
				cycle})  // collections are rendered as comma-separated lists
			: ctx.createSuccessStatus();
	}

	/**
	 * Determines whether a <code>classifier</code> has a generalization cycle.
	 * As a side-effect, the <code>cycle</code> list accumulates the members
	 * of the cycle.
	 * 
	 * @param classifier a classifier to test for generalization cycle
	 * @param cycle collects the elements traversed in the
	 *     <code>classifier</code>'s ancestry.  If a cycle is detected, then
	 *     this list will contain it
	 * @return <code>true</code> if a cycle is detected and stored in
	 *     the <code>cycle</code> parameter; <code>false</code>, otherwise
	 */
	private boolean hasCycle(Classifier classifier, List cycle) {
		if (cycle.contains(classifier)) {
			// detected the cycle
			return true;
		}

		cycle.add(0, classifier);
		Collection parents = classifier.parents();

		for (Iterator iter = parents.iterator(); iter.hasNext();) {
			if (hasCycle((Classifier) iter.next(), cycle)) {
				return true;
			}
		}
		
		// this classifier did not cause a cycle, so remove it again from the
		//   list of tentative cycle members
		cycle.remove(0);
		
		return false;
	}
}
