/*
 *+--------------------------------------------------------------------------------------+
 *| Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms and |
 *| conditions in the IBM International Program License Agreement.						 |	
 *|  Copyright IBM Corporation 2003, 2004. All Rights Reserved.						 |
 *+--------------------------------------------------------------------------------------+
 */


/*
 * File: CreateFactoryClassRule.java
 */

package com.ibm.xtools.transform.samples.modeltomodel.classtoservice.rules;

import org.eclipse.uml2.Class;
import org.eclipse.uml2.Interface;
import org.eclipse.uml2.Operation;
import org.eclipse.uml2.Package;
import org.eclipse.uml2.Parameter;
import org.eclipse.uml2.ParameterDirectionKind;
import org.eclipse.uml2.Stereotype;
import org.eclipse.uml2.UML2Package;
import org.eclipse.uml2.Usage;

import com.ibm.xtools.transform.core.AbstractRule;
import com.ibm.xtools.transform.core.ITransformContext;
import com.ibm.xtools.transform.samples.modeltomodel.IsTopLevelClassCondition;
import com.ibm.xtools.transform.samples.modeltomodel.ModelUtility;
import com.ibm.xtools.transform.samples.modeltomodel.l10n.ResourceManager;
import com.ibm.xtools.transform.uml2.IsElementKindCondition;

/**
 * A rule that accepts a top level class as the source and creates a factory class and
 * creates the necessary usage relationship.
 * 
 * A top level class is a class that is contained directly by a package or a model.
 *  
 */
public class CreateFactoryClassRule
	extends AbstractRule {

	public static final String ID = "classtoservice.create.factory.rule"; //$NON-NLS-1$

	public static final String NAME = ResourceManager
		.getString("CreateFactoryClassRule.name"); //$NON-NLS-1$

	/**
     * Default constructor. 
     * 
     * Initializes the id and the name of the rule to
     * their default values. It also initializes it to accept only
     * a top level UML2 class as source.  A top level class is a class that
     * is contained directly by a package or a model.
     */
    public CreateFactoryClassRule() {
    	super(ID, NAME);
    	setAcceptCondition(new IsElementKindCondition(UML2Package.eINSTANCE
			.getClass_()).AND(new IsTopLevelClassCondition()));
    }
	
    /**
     * Constructor.
     * 
     * Creates an instance of the rule and initializes it to accept only
     * a top level UML2 class as source.  A top level class is a class that
     * is contained directly by a package or a model.
     * 
	 * @param id
	 *            Unique id of the rule
	 * @param name
	 *            Readable name of the rule
	 */
	public CreateFactoryClassRule(String id, String name) {
		super(id, name);
		setAcceptCondition(new IsElementKindCondition(UML2Package.eINSTANCE
			.getClass_()).AND(new IsTopLevelClassCondition()));
	}

	/*
	 * Factory Rule: Create the factory method with a return parameter.
	 * 
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.xtools.transform.uml2.internal.AbstractUMLRule#doCreate(com.ibm.xtools.transform.core.internal.engine.ITransformContext)
	 */
	protected Object createTarget(ITransformContext ruleContext) {
		UML2Package uml = UML2Package.eINSTANCE;
		Class cls = (Class) ruleContext.getSource();
		String className = cls.getName();

		Package pkg = (Package) ruleContext.getPropertyValue("targetPackage"); //$NON-NLS-1$
		Interface intface = ModelUtility.getInterfaceByName(pkg,
			"I" + className); //$NON-NLS-1$
		Class impl = ModelUtility.getClassByName(pkg, className + "Impl"); //$NON-NLS-1$
		Class factory = null;

		if (pkg != null && intface != null) {
			String factoryClassName = className + "Factory"; //$NON-NLS-1$
			factory = ModelUtility.getClassByName(pkg, factoryClassName);

			if (factory == null) {
				factory = (Class) pkg.createOwnedMember(uml.getClass_());
				factory.setName(factoryClassName);

				Operation factoryMethod = factory.createOwnedOperation(uml.getOperation());
				factoryMethod.setName("create"); //$NON-NLS-1$
				factoryMethod.setIsStatic(true);
				Parameter createReturn = factoryMethod.createReturnResult(uml.getParameter());
				createReturn.setDirection(ParameterDirectionKind.RETURN_LITERAL);
				createReturn.setType(intface);

				// Set up an <<Instantiate>> dependency between the factory and
				// the
				// implementation class
				Usage u = (Usage) pkg.createOwnedMember(uml.getUsage());
				u.getClients().add(factory);
				u.getSuppliers().add(impl);
				Stereotype s = u.getApplicableStereotype("Basic::Instantiate"); //$NON-NLS-1$
				u.apply(s);
			}
		}

		System.out.println(ID + " is executed"); //$NON-NLS-1$

		return factory;
	}

}